@@ -1,17 +1,14 @@
-import aiohttp
 import asyncio
 
-import aiohttp.http_exceptions
 from botocore.endpoint import EndpointCreator, Endpoint, DEFAULT_TIMEOUT, \
     MAX_POOL_CONNECTIONS, logger, history_recorder, create_request_object, \
-    is_valid_ipv6_endpoint_url, is_valid_endpoint_url
-from botocore.exceptions import ConnectionClosedError
+    is_valid_ipv6_endpoint_url, is_valid_endpoint_url, HTTPClientError
 from botocore.hooks import first_non_none_response
 from urllib3.response import HTTPHeaderDict
 
 from aiobotocore.httpsession import AIOHTTPSession
 from aiobotocore.response import StreamingBody
-from aiobotocore._endpoint_helpers import ClientResponseProxy  # noqa: F401, E501 lgtm [py/unused-import]
+from aiobotocore.httpchecksum import handle_checksum_body
 
 
 async def convert_to_response_dict(http_response, operation_model):
@@ -37,21 +34,21 @@ async def convert_to_response_dict(http_response, operation_model):
         # aiohttp's CIMultiDict camel cases the headers :(
         'headers': HTTPHeaderDict(
             {k.decode('utf-8').lower(): v.decode('utf-8')
-             for k, v in http_response.raw_headers}),
+             for k, v in http_response.raw.raw_headers}),
         'status_code': http_response.status_code,
         'context': {
             'operation_name': operation_model.name,
         }
     }
     if response_dict['status_code'] >= 300:
-        response_dict['body'] = await http_response.read()
+        response_dict['body'] = await http_response.content
     elif operation_model.has_event_stream_output:
         response_dict['body'] = http_response.raw
     elif operation_model.has_streaming_output:
         length = response_dict['headers'].get('content-length')
         response_dict['body'] = StreamingBody(http_response.raw, length)
     else:
-        response_dict['body'] = await http_response.read()
+        response_dict['body'] = await http_response.content
     return response_dict
 
 
@@ -74,6 +71,8 @@ class AioEndpoint(Endpoint):
 
     async def _send_request(self, request_dict, operation_model):
         attempts = 1
+        context = request_dict['context']
+        self._update_retries_context(context, attempts)
         request = await self.create_request(request_dict, operation_model)
         context = request_dict['context']
         success_response, exception = await self._get_response(
@@ -82,6 +81,9 @@ class AioEndpoint(Endpoint):
                                       request_dict, success_response,
                                       exception):
             attempts += 1
+            self._update_retries_context(
+                context, attempts, success_response
+            )
             # If there is a stream associated with the request, we need
             # to reset it before attempting to send the request again.
             # This will ensure that we resend the entire contents of the
@@ -109,7 +111,7 @@ class AioEndpoint(Endpoint):
         # If an exception occurs then the success_response is None.
         # If no exception occurs then exception is None.
         success_response, exception = await self._do_get_response(
-            request, operation_model)
+            request, operation_model, context)
         kwargs_to_emit = {
             'response_dict': None,
             'parsed_response': None,
@@ -127,7 +129,7 @@ class AioEndpoint(Endpoint):
                 service_id, operation_model.name), **kwargs_to_emit)
         return success_response, exception
 
-    async def _do_get_response(self, request, operation_model):
+    async def _do_get_response(self, request, operation_model, context):
         try:
             logger.debug("Sending http request: %s", request)
             history_recorder.record('HTTP_REQUEST', {
@@ -145,13 +147,8 @@ class AioEndpoint(Endpoint):
             http_response = first_non_none_response(responses)
             if http_response is None:
                 http_response = await self._send(request)
-        except aiohttp.ClientConnectionError as e:
-            e.request = request  # botocore expects the request property
+        except HTTPClientError as e:
             return None, e
-        except aiohttp.http_exceptions.BadStatusLine:
-            better_exception = ConnectionClosedError(
-                endpoint_url=request.url, request=request)
-            return None, better_exception
         except Exception as e:
             logger.debug("Exception received when sending HTTP request.",
                          exc_info=True)
@@ -160,6 +157,9 @@ class AioEndpoint(Endpoint):
         # This returns the http_response and the parsed_data.
         response_dict = await convert_to_response_dict(http_response,
                                                        operation_model)
+        await handle_checksum_body(
+            http_response, response_dict, context, operation_model,
+        )
 
         http_response_record_dict = response_dict.copy()
         http_response_record_dict['streaming'] = \
